# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, double, integer
from .validators.batch import (
    validate_allocation_strategy,
    validate_environment_state,
    validate_launch_template_specification,
    validate_queue_state,
)


class Ec2ConfigurationObject(AWSProperty):
    """
    `Ec2ConfigurationObject <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-ec2configurationobject.html>`__
    """

    props: PropsDictType = {
        "ImageIdOverride": (str, False),
        "ImageKubernetesVersion": (str, False),
        "ImageType": (str, True),
    }


class LaunchTemplateSpecification(AWSProperty):
    """
    `LaunchTemplateSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-launchtemplatespecification.html>`__
    """

    props: PropsDictType = {
        "LaunchTemplateId": (str, False),
        "LaunchTemplateName": (str, False),
        "Version": (str, False),
    }

    def validate(self):
        validate_launch_template_specification(self)


class ComputeResources(AWSProperty):
    """
    `ComputeResources <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html>`__
    """

    props: PropsDictType = {
        "AllocationStrategy": (validate_allocation_strategy, False),
        "BidPercentage": (integer, False),
        "DesiredvCpus": (integer, False),
        "Ec2Configuration": ([Ec2ConfigurationObject], False),
        "Ec2KeyPair": (str, False),
        "ImageId": (str, False),
        "InstanceRole": (str, False),
        "InstanceTypes": ([str], False),
        "LaunchTemplate": (LaunchTemplateSpecification, False),
        "MaxvCpus": (integer, True),
        "MinvCpus": (integer, False),
        "PlacementGroup": (str, False),
        "SecurityGroupIds": ([str], False),
        "SpotIamFleetRole": (str, False),
        "Subnets": ([str], True),
        "Tags": (dict, False),
        "Type": (str, True),
        "UpdateToLatestImageVersion": (boolean, False),
    }


class EksConfiguration(AWSProperty):
    """
    `EksConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-eksconfiguration.html>`__
    """

    props: PropsDictType = {
        "EksClusterArn": (str, True),
        "KubernetesNamespace": (str, True),
    }


class UpdatePolicy(AWSProperty):
    """
    `UpdatePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-updatepolicy.html>`__
    """

    props: PropsDictType = {
        "JobExecutionTimeoutMinutes": (integer, False),
        "TerminateJobsOnUpdate": (boolean, False),
    }


class ComputeEnvironment(AWSObject):
    """
    `ComputeEnvironment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html>`__
    """

    resource_type = "AWS::Batch::ComputeEnvironment"

    props: PropsDictType = {
        "ComputeEnvironmentName": (str, False),
        "ComputeResources": (ComputeResources, False),
        "EksConfiguration": (EksConfiguration, False),
        "ReplaceComputeEnvironment": (boolean, False),
        "ServiceRole": (str, False),
        "State": (validate_environment_state, False),
        "Tags": (dict, False),
        "Type": (str, True),
        "UnmanagedvCpus": (integer, False),
        "UpdatePolicy": (UpdatePolicy, False),
    }


class Environment(AWSProperty):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-environment.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Value": (str, False),
    }


class EphemeralStorage(AWSProperty):
    """
    `EphemeralStorage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties-ephemeralstorage.html>`__
    """

    props: PropsDictType = {
        "SizeInGiB": (integer, True),
    }


class FargatePlatformConfiguration(AWSProperty):
    """
    `FargatePlatformConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties-fargateplatformconfiguration.html>`__
    """

    props: PropsDictType = {
        "PlatformVersion": (str, False),
    }


class Device(AWSProperty):
    """
    `Device <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-device.html>`__
    """

    props: PropsDictType = {
        "ContainerPath": (str, False),
        "HostPath": (str, False),
        "Permissions": ([str], False),
    }


class Tmpfs(AWSProperty):
    """
    `Tmpfs <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-tmpfs.html>`__
    """

    props: PropsDictType = {
        "ContainerPath": (str, True),
        "MountOptions": ([str], False),
        "Size": (integer, True),
    }


class LinuxParameters(AWSProperty):
    """
    `LinuxParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties-linuxparameters.html>`__
    """

    props: PropsDictType = {
        "Devices": ([Device], False),
        "InitProcessEnabled": (boolean, False),
        "MaxSwap": (integer, False),
        "SharedMemorySize": (integer, False),
        "Swappiness": (integer, False),
        "Tmpfs": ([Tmpfs], False),
    }


class Secret(AWSProperty):
    """
    `Secret <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-secret.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "ValueFrom": (str, True),
    }


class LogConfiguration(AWSProperty):
    """
    `LogConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties-logconfiguration.html>`__
    """

    props: PropsDictType = {
        "LogDriver": (str, True),
        "Options": (dict, False),
        "SecretOptions": ([Secret], False),
    }


class MountPoints(AWSProperty):
    """
    `MountPoints <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-mountpoints.html>`__
    """

    props: PropsDictType = {
        "ContainerPath": (str, False),
        "ReadOnly": (boolean, False),
        "SourceVolume": (str, False),
    }


class NetworkConfiguration(AWSProperty):
    """
    `NetworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties-networkconfiguration.html>`__
    """

    props: PropsDictType = {
        "AssignPublicIp": (str, False),
    }


class ResourceRequirement(AWSProperty):
    """
    `ResourceRequirement <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-resourcerequirement.html>`__
    """

    props: PropsDictType = {
        "Type": (str, False),
        "Value": (str, False),
    }


class RuntimePlatform(AWSProperty):
    """
    `RuntimePlatform <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties-runtimeplatform.html>`__
    """

    props: PropsDictType = {
        "CpuArchitecture": (str, False),
        "OperatingSystemFamily": (str, False),
    }


class Ulimit(AWSProperty):
    """
    `Ulimit <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ulimit.html>`__
    """

    props: PropsDictType = {
        "HardLimit": (integer, True),
        "Name": (str, True),
        "SoftLimit": (integer, True),
    }


class AuthorizationConfig(AWSProperty):
    """
    `AuthorizationConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-authorizationconfig.html>`__
    """

    props: PropsDictType = {
        "AccessPointId": (str, False),
        "Iam": (str, False),
    }


class EfsVolumeConfiguration(AWSProperty):
    """
    `EfsVolumeConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-efsvolumeconfiguration.html>`__
    """

    props: PropsDictType = {
        "AuthorizationConfig": (AuthorizationConfig, False),
        "FileSystemId": (str, True),
        "RootDirectory": (str, False),
        "TransitEncryption": (str, False),
        "TransitEncryptionPort": (integer, False),
    }


class VolumesHost(AWSProperty):
    """
    `VolumesHost <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-volumeshost.html>`__
    """

    props: PropsDictType = {
        "SourcePath": (str, False),
    }


class Volumes(AWSProperty):
    """
    `Volumes <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-volumes.html>`__
    """

    props: PropsDictType = {
        "EfsVolumeConfiguration": (EfsVolumeConfiguration, False),
        "Host": (VolumesHost, False),
        "Name": (str, False),
    }


class ContainerProperties(AWSProperty):
    """
    `ContainerProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html>`__
    """

    props: PropsDictType = {
        "Command": ([str], False),
        "Environment": ([Environment], False),
        "EphemeralStorage": (EphemeralStorage, False),
        "ExecutionRoleArn": (str, False),
        "FargatePlatformConfiguration": (FargatePlatformConfiguration, False),
        "Image": (str, True),
        "InstanceType": (str, False),
        "JobRoleArn": (str, False),
        "LinuxParameters": (LinuxParameters, False),
        "LogConfiguration": (LogConfiguration, False),
        "Memory": (integer, False),
        "MountPoints": ([MountPoints], False),
        "NetworkConfiguration": (NetworkConfiguration, False),
        "Privileged": (boolean, False),
        "ReadonlyRootFilesystem": (boolean, False),
        "ResourceRequirements": ([ResourceRequirement], False),
        "RuntimePlatform": (RuntimePlatform, False),
        "Secrets": ([Secret], False),
        "Ulimits": ([Ulimit], False),
        "User": (str, False),
        "Vcpus": (integer, False),
        "Volumes": ([Volumes], False),
    }


class EksContainerEnvironmentVariable(AWSProperty):
    """
    `EksContainerEnvironmentVariable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekscontainerenvironmentvariable.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "Value": (str, False),
    }


class EksContainerResourceRequirements(AWSProperty):
    """
    `EksContainerResourceRequirements <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekscontainerresourcerequirements.html>`__
    """

    props: PropsDictType = {
        "Limits": (dict, False),
        "Requests": (dict, False),
    }


class EksContainerSecurityContext(AWSProperty):
    """
    `EksContainerSecurityContext <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekscontainersecuritycontext.html>`__
    """

    props: PropsDictType = {
        "Privileged": (boolean, False),
        "ReadOnlyRootFilesystem": (boolean, False),
        "RunAsGroup": (integer, False),
        "RunAsNonRoot": (boolean, False),
        "RunAsUser": (integer, False),
    }


class EksContainerVolumeMount(AWSProperty):
    """
    `EksContainerVolumeMount <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekscontainervolumemount.html>`__
    """

    props: PropsDictType = {
        "MountPath": (str, False),
        "Name": (str, False),
        "ReadOnly": (boolean, False),
    }


class EksContainer(AWSProperty):
    """
    `EksContainer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekscontainer.html>`__
    """

    props: PropsDictType = {
        "Args": ([str], False),
        "Command": ([str], False),
        "Env": ([EksContainerEnvironmentVariable], False),
        "Image": (str, True),
        "ImagePullPolicy": (str, False),
        "Name": (str, False),
        "Resources": (EksContainerResourceRequirements, False),
        "SecurityContext": (EksContainerSecurityContext, False),
        "VolumeMounts": ([EksContainerVolumeMount], False),
    }


class EksEmptyDir(AWSProperty):
    """
    `EksEmptyDir <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-eksemptydir.html>`__
    """

    props: PropsDictType = {
        "Medium": (str, False),
        "SizeLimit": (str, False),
    }


class EksHostPath(AWSProperty):
    """
    `EksHostPath <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekshostpath.html>`__
    """

    props: PropsDictType = {
        "Path": (str, False),
    }


class EksSecret(AWSProperty):
    """
    `EksSecret <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ekssecret.html>`__
    """

    props: PropsDictType = {
        "Optional": (boolean, False),
        "SecretName": (str, True),
    }


class EksVolume(AWSProperty):
    """
    `EksVolume <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-eksvolume.html>`__
    """

    props: PropsDictType = {
        "EmptyDir": (EksEmptyDir, False),
        "HostPath": (EksHostPath, False),
        "Name": (str, True),
        "Secret": (EksSecret, False),
    }


class Metadata(AWSProperty):
    """
    `Metadata <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-podproperties-metadata.html>`__
    """

    props: PropsDictType = {
        "Labels": (dict, False),
    }


class PodProperties(AWSProperty):
    """
    `PodProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-podproperties.html>`__
    """

    props: PropsDictType = {
        "Containers": ([EksContainer], False),
        "DnsPolicy": (str, False),
        "HostNetwork": (boolean, False),
        "Metadata": (Metadata, False),
        "ServiceAccountName": (str, False),
        "Volumes": ([EksVolume], False),
    }


class EksProperties(AWSProperty):
    """
    `EksProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-eksproperties.html>`__
    """

    props: PropsDictType = {
        "PodProperties": (PodProperties, False),
    }


class NodeRangeProperty(AWSProperty):
    """
    `NodeRangeProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-noderangeproperty.html>`__
    """

    props: PropsDictType = {
        "Container": (ContainerProperties, False),
        "TargetNodes": (str, True),
    }


class NodeProperties(AWSProperty):
    """
    `NodeProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-nodeproperties.html>`__
    """

    props: PropsDictType = {
        "MainNode": (integer, True),
        "NodeRangeProperties": ([NodeRangeProperty], True),
        "NumNodes": (integer, True),
    }


class EvaluateOnExit(AWSProperty):
    """
    `EvaluateOnExit <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-evaluateonexit.html>`__
    """

    props: PropsDictType = {
        "Action": (str, True),
        "OnExitCode": (str, False),
        "OnReason": (str, False),
        "OnStatusReason": (str, False),
    }


class RetryStrategy(AWSProperty):
    """
    `RetryStrategy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-retrystrategy.html>`__
    """

    props: PropsDictType = {
        "Attempts": (integer, False),
        "EvaluateOnExit": ([EvaluateOnExit], False),
    }


class Timeout(AWSProperty):
    """
    `Timeout <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-timeout.html>`__
    """

    props: PropsDictType = {
        "AttemptDurationSeconds": (integer, False),
    }


class JobDefinition(AWSObject):
    """
    `JobDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html>`__
    """

    resource_type = "AWS::Batch::JobDefinition"

    props: PropsDictType = {
        "ContainerProperties": (ContainerProperties, False),
        "EksProperties": (EksProperties, False),
        "JobDefinitionName": (str, False),
        "NodeProperties": (NodeProperties, False),
        "Parameters": (dict, False),
        "PlatformCapabilities": ([str], False),
        "PropagateTags": (boolean, False),
        "RetryStrategy": (RetryStrategy, False),
        "SchedulingPriority": (integer, False),
        "Tags": (dict, False),
        "Timeout": (Timeout, False),
        "Type": (str, True),
    }


class ComputeEnvironmentOrder(AWSProperty):
    """
    `ComputeEnvironmentOrder <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobqueue-computeenvironmentorder.html>`__
    """

    props: PropsDictType = {
        "ComputeEnvironment": (str, True),
        "Order": (integer, True),
    }


class JobQueue(AWSObject):
    """
    `JobQueue <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html>`__
    """

    resource_type = "AWS::Batch::JobQueue"

    props: PropsDictType = {
        "ComputeEnvironmentOrder": ([ComputeEnvironmentOrder], True),
        "JobQueueName": (str, False),
        "Priority": (integer, True),
        "SchedulingPolicyArn": (str, False),
        "State": (validate_queue_state, False),
        "Tags": (dict, False),
    }


class ShareAttributes(AWSProperty):
    """
    `ShareAttributes <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-schedulingpolicy-shareattributes.html>`__
    """

    props: PropsDictType = {
        "ShareIdentifier": (str, False),
        "WeightFactor": (double, False),
    }


class FairsharePolicy(AWSProperty):
    """
    `FairsharePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-schedulingpolicy-fairsharepolicy.html>`__
    """

    props: PropsDictType = {
        "ComputeReservation": (double, False),
        "ShareDecaySeconds": (double, False),
        "ShareDistribution": ([ShareAttributes], False),
    }


class SchedulingPolicy(AWSObject):
    """
    `SchedulingPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-schedulingpolicy.html>`__
    """

    resource_type = "AWS::Batch::SchedulingPolicy"

    props: PropsDictType = {
        "FairsharePolicy": (FairsharePolicy, False),
        "Name": (str, False),
        "Tags": (dict, False),
    }



import unittest

from troposphere import codebuild


class TestCodeBuild(unittest.TestCase):
    def test_linux_environment(self):
        environment = codebuild.Environment(
            ComputeType="BUILD_GENERAL1_SMALL",
            Image="aws/codebuild/ubuntu-base:14.04",
            Type="LINUX_CONTAINER",
        )
        environment.to_dict()

    def test_windows_environment(self):
        environment = codebuild.Environment(
            ComputeType="BUILD_GENERAL1_LARGE",
            Image="aws/codebuild/windows-base:1.0",
            Type="WINDOWS_CONTAINER",
        )
        environment.to_dict()

    def test_arm_environment(self):
        environment = codebuild.Environment(
            ComputeType="BUILD_GENERAL1_LARGE",
            Image="aws/codebuild/amazonlinux2-aarch64-standard:1.0",
            Type="ARM_CONTAINER",
        )
        environment.to_dict()

    def test_linux_gpu_environment(self):
        environment = codebuild.Environment(
            ComputeType="BUILD_GENERAL1_LARGE",
            Image="aws/codebuild/standard:4.0",
            Type="LINUX_GPU_CONTAINER",
        )
        environment.to_dict()

    def test_source_codepipeline(self):
        source = codebuild.Source(Type="CODEPIPELINE")
        source.to_dict()


class TestCodeBuildFilters(unittest.TestCase):
    def test_filter(self):
        wh = codebuild.WebhookFilter
        codebuild.ProjectTriggers(
            FilterGroups=[
                [wh(Type="EVENT", Pattern="PULL_REQUEST_CREATED")],
                [wh(Type="EVENT", Pattern="PULL_REQUEST_CREATED")],
            ]
        ).validate()

    def test_filter_no_filtergroup(self):
        codebuild.ProjectTriggers(Webhook=True).validate()
        # Technically this is valid, not sure why you would want this though?
        codebuild.ProjectTriggers().validate()

    def test_filter_not_list(self):
        match = "<class 'int'>, expected <class 'list'>"
        with self.assertRaisesRegex(TypeError, match):
            codebuild.ProjectTriggers(FilterGroups=42).validate()

    def test_filter_element_not_a_list(self):
        wh = codebuild.WebhookFilter
        match = "is <class 'str'>, expected <class 'list'>"
        with self.assertRaisesRegex(TypeError, match):
            codebuild.ProjectTriggers(
                FilterGroups=[
                    [wh(Type="EVENT", Pattern="PULL_REQUEST_CREATED")],
                    "not a list",
                    [wh(Type="EVENT", Pattern="PULL_REQUEST_CREATED")],
                ]
            ).validate()

    def test_filter_fail(self):
        wh = codebuild.WebhookFilter
        match = "<class 'NoneType'>"
        with self.assertRaisesRegex(TypeError, match):
            codebuild.ProjectTriggers(
                FilterGroups=[
                    [wh(Type="EVENT", Pattern="PULL_REQUEST_CREATED")],
                    [wh(Type="EVENT", Pattern="PULL_REQUEST_CREATED")],
                    [None],
                ]
            ).validate()


class TestCodeBuildRegistryCredential(unittest.TestCase):
    def test_registrycredential_provider_bad_value(self):
        with self.assertRaisesRegex(ValueError, "CredentialProvider must be one of"):
            codebuild.RegistryCredential(
                Credential="Foo",
                CredentialProvider="SECRETS",
            )

    def test_registrycredential(self):
        codebuild.RegistryCredential(
            Credential="Foo",
            CredentialProvider="SECRETS_MANAGER",
        )


class TestCodeBuildProjectFileSystemLocation(unittest.TestCase):
    def test_projectfilesystemlocation_type_bad_value(self):
        with self.assertRaisesRegex(ValueError, "Type must be one of"):
            codebuild.ProjectFileSystemLocation(
                Identifier="foo",
                Location="bar",
                MountPoint="baz",
                Type="EF",
            )

    def test_projectfilesystemlocation(self):
        codebuild.ProjectFileSystemLocation(
            Identifier="foo",
            Location="bar",
            MountPoint="baz",
            Type="EFS",
        )



import troposphere.ec2 as ec2
from troposphere import FindInMap, GetAtt, Output, Parameter, Ref, Template
from troposphere.ec2 import NO_DEVICE

template = Template()

keyname_param = template.add_parameter(
    Parameter(
        "KeyName",
        Description="Name of an existing EC2 KeyPair to enable SSH "
        "access to the instance",
        Type="String",
    )
)

template.add_mapping(
    "RegionMap",
    {
        "ap-northeast-1": {"AMI": "ami-6959870f"},
        "ap-northeast-2": {"AMI": "ami-08d77266"},
        "ap-south-1": {"AMI": "ami-50591a3f"},
        "ap-southeast-1": {"AMI": "ami-d9dca7ba"},
        "ap-southeast-2": {"AMI": "ami-02ad4060"},
        "ca-central-1": {"AMI": "ami-13e45c77"},
        "eu-central-1": {"AMI": "ami-e613ac89"},
        "eu-west-1": {"AMI": "ami-eed00d97"},
        "eu-west-2": {"AMI": "ami-ba5f42de"},
        "sa-east-1": {"AMI": "ami-1ca7d970"},
        "us-east-1": {"AMI": "ami-bcdc16c6"},
        "us-east-2": {"AMI": "ami-49426e2c"},
        "us-west-1": {"AMI": "ami-1b17257b"},
        "us-west-2": {"AMI": "ami-19e92861"},
    },
)

ec2_instance = template.add_resource(
    ec2.Instance(
        "Ec2Instance",
        ImageId=FindInMap("RegionMap", Ref("AWS::Region"), "AMI"),
        InstanceType="t2.micro",
        KeyName=Ref(keyname_param),
        SecurityGroups=["default"],
        BlockDeviceMappings=[
            ec2.BlockDeviceMapping(DeviceName="/dev/sdb", NoDevice=NO_DEVICE),
            ec2.BlockDeviceMapping(DeviceName="/dev/sdc", NoDevice=NO_DEVICE),
        ],
    )
)

template.add_output(
    [
        Output(
            "InstanceId",
            Description="InstanceId of the newly created EC2 instance",
            Value=Ref(ec2_instance),
        ),
        Output(
            "AZ",
            Description="Availability Zone of the newly created EC2 instance",
            Value=GetAtt(ec2_instance, "AvailabilityZone"),
        ),
        Output(
            "PublicIP",
            Description="Public IP address of the newly created EC2 instance",
            Value=GetAtt(ec2_instance, "PublicIp"),
        ),
        Output(
            "PrivateIP",
            Description="Private IP address of the newly created EC2 instance",
            Value=GetAtt(ec2_instance, "PrivateIp"),
        ),
        Output(
            "PublicDNS",
            Description="Public DNSName of the newly created EC2 instance",
            Value=GetAtt(ec2_instance, "PublicDnsName"),
        ),
        Output(
            "PrivateDNS",
            Description="Private DNSName of the newly created EC2 instance",
            Value=GetAtt(ec2_instance, "PrivateDnsName"),
        ),
    ]
)

print(template.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double
from .validators.rekognition import validate_PolygonRegionsOfInterest


class Collection(AWSObject):
    """
    `Collection <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rekognition-collection.html>`__
    """

    resource_type = "AWS::Rekognition::Collection"

    props: PropsDictType = {
        "CollectionId": (str, True),
        "Tags": (Tags, False),
    }


class Project(AWSObject):
    """
    `Project <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rekognition-project.html>`__
    """

    resource_type = "AWS::Rekognition::Project"

    props: PropsDictType = {
        "ProjectName": (str, True),
    }


class BoundingBox(AWSProperty):
    """
    `BoundingBox <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-boundingbox.html>`__
    """

    props: PropsDictType = {
        "Height": (double, True),
        "Left": (double, True),
        "Top": (double, True),
        "Width": (double, True),
    }


class ConnectedHomeSettings(AWSProperty):
    """
    `ConnectedHomeSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-connectedhomesettings.html>`__
    """

    props: PropsDictType = {
        "Labels": ([str], True),
        "MinConfidence": (double, False),
    }


class DataSharingPreference(AWSProperty):
    """
    `DataSharingPreference <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-datasharingpreference.html>`__
    """

    props: PropsDictType = {
        "OptIn": (boolean, True),
    }


class FaceSearchSettings(AWSProperty):
    """
    `FaceSearchSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-facesearchsettings.html>`__
    """

    props: PropsDictType = {
        "CollectionId": (str, True),
        "FaceMatchThreshold": (double, False),
    }


class KinesisDataStream(AWSProperty):
    """
    `KinesisDataStream <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-kinesisdatastream.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, True),
    }


class KinesisVideoStream(AWSProperty):
    """
    `KinesisVideoStream <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-kinesisvideostream.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, True),
    }


class NotificationChannel(AWSProperty):
    """
    `NotificationChannel <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-notificationchannel.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, True),
    }


class S3Destination(AWSProperty):
    """
    `S3Destination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rekognition-streamprocessor-s3destination.html>`__
    """

    props: PropsDictType = {
        "BucketName": (str, True),
        "ObjectKeyPrefix": (str, False),
    }


class StreamProcessor(AWSObject):
    """
    `StreamProcessor <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rekognition-streamprocessor.html>`__
    """

    resource_type = "AWS::Rekognition::StreamProcessor"

    props: PropsDictType = {
        "BoundingBoxRegionsOfInterest": ([BoundingBox], False),
        "ConnectedHomeSettings": (ConnectedHomeSettings, False),
        "DataSharingPreference": (DataSharingPreference, False),
        "FaceSearchSettings": (FaceSearchSettings, False),
        "KinesisDataStream": (KinesisDataStream, False),
        "KinesisVideoStream": (KinesisVideoStream, True),
        "KmsKeyId": (str, False),
        "Name": (str, False),
        "NotificationChannel": (NotificationChannel, False),
        "PolygonRegionsOfInterest": (validate_PolygonRegionsOfInterest, False),
        "RoleArn": (str, True),
        "S3Destination": (S3Destination, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import integer


class InputNameConfiguration(AWSProperty):
    """
    `InputNameConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lookoutequipment-inferencescheduler-inputnameconfiguration.html>`__
    """

    props: PropsDictType = {
        "ComponentTimestampDelimiter": (str, False),
        "TimestampFormat": (str, False),
    }


class S3InputConfiguration(AWSProperty):
    """
    `S3InputConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lookoutequipment-inferencescheduler-s3inputconfiguration.html>`__
    """

    props: PropsDictType = {
        "Bucket": (str, True),
        "Prefix": (str, False),
    }


class DataInputConfiguration(AWSProperty):
    """
    `DataInputConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lookoutequipment-inferencescheduler-datainputconfiguration.html>`__
    """

    props: PropsDictType = {
        "InferenceInputNameConfiguration": (InputNameConfiguration, False),
        "InputTimeZoneOffset": (str, False),
        "S3InputConfiguration": (S3InputConfiguration, True),
    }


class S3OutputConfiguration(AWSProperty):
    """
    `S3OutputConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lookoutequipment-inferencescheduler-s3outputconfiguration.html>`__
    """

    props: PropsDictType = {
        "Bucket": (str, True),
        "Prefix": (str, False),
    }


class DataOutputConfiguration(AWSProperty):
    """
    `DataOutputConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lookoutequipment-inferencescheduler-dataoutputconfiguration.html>`__
    """

    props: PropsDictType = {
        "KmsKeyId": (str, False),
        "S3OutputConfiguration": (S3OutputConfiguration, True),
    }


class InferenceScheduler(AWSObject):
    """
    `InferenceScheduler <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lookoutequipment-inferencescheduler.html>`__
    """

    resource_type = "AWS::LookoutEquipment::InferenceScheduler"

    props: PropsDictType = {
        "DataDelayOffsetInMinutes": (integer, False),
        "DataInputConfiguration": (DataInputConfiguration, True),
        "DataOutputConfiguration": (DataOutputConfiguration, True),
        "DataUploadFrequency": (str, True),
        "InferenceSchedulerName": (str, False),
        "ModelName": (str, True),
        "RoleArn": (str, True),
        "ServerSideKmsKeyId": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import integer


class MaintenanceWindow(AWSProperty):
    """
    `MaintenanceWindow <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-workspacesthinclient-environment-maintenancewindow.html>`__
    """

    props: PropsDictType = {
        "ApplyTimeOf": (str, False),
        "DaysOfTheWeek": ([str], False),
        "EndTimeHour": (integer, False),
        "EndTimeMinute": (integer, False),
        "StartTimeHour": (integer, False),
        "StartTimeMinute": (integer, False),
        "Type": (str, True),
    }


class Environment(AWSObject):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspacesthinclient-environment.html>`__
    """

    resource_type = "AWS::WorkSpacesThinClient::Environment"

    props: PropsDictType = {
        "DesiredSoftwareSetId": (str, False),
        "DesktopArn": (str, True),
        "DesktopEndpoint": (str, False),
        "KmsKeyArn": (str, False),
        "MaintenanceWindow": (MaintenanceWindow, False),
        "Name": (str, False),
        "SoftwareSetUpdateMode": (str, False),
        "SoftwareSetUpdateSchedule": (str, False),
        "Tags": (Tags, False),
    }



import unittest

from troposphere import Template
from troposphere.opsworks import Stack


class TestOpsWorksStack(unittest.TestCase):
    def test_nosubnet(self):
        stack = Stack(
            "mystack",
            VpcId="myvpcid",
        )
        with self.assertRaises(ValueError):
            stack.validate()

    def test_stack(self):
        stack = Stack(
            "mystack",
            VpcId="myvpcid",
            DefaultSubnetId="subnetid",
        )
        self.assertIsNone(stack.validate())

    def test_no_required(self):
        stack = Stack(
            "mystack",
        )
        t = Template()
        t.add_resource(stack)
        with self.assertRaises(ValueError):
            t.to_json()

    def test_required(self):
        stack = Stack(
            "mystack",
            DefaultInstanceProfileArn="instancearn",
            Name="myopsworksname",
            ServiceRoleArn="arn",
        )
        t = Template()
        t.add_resource(stack)
        t.to_json()

    def test_custom_json(self):
        stack = Stack(
            "mystack",
            DefaultInstanceProfileArn="instancearn",
            Name="myopsworksname",
            ServiceRoleArn="arn",
        )

        # Test dict works
        t = Template()
        stack.CustomJson = {"foo": "bar"}
        t.add_resource(stack)
        t.to_json()

        # Test json string works
        t = Template()
        stack.CustomJson = '{"foo": "bar"}'
        t.add_resource(stack)
        t.to_json()

        # Test boolean fails
        with self.assertRaises(TypeError):
            stack.CustomJson = True


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import double, integer


class ConditionBasedCollectionScheme(AWSProperty):
    """
    `ConditionBasedCollectionScheme <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-conditionbasedcollectionscheme.html>`__
    """

    props: PropsDictType = {
        "ConditionLanguageVersion": (integer, False),
        "Expression": (str, True),
        "MinimumTriggerIntervalMs": (double, False),
        "TriggerMode": (str, False),
    }


class TimeBasedCollectionScheme(AWSProperty):
    """
    `TimeBasedCollectionScheme <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-timebasedcollectionscheme.html>`__
    """

    props: PropsDictType = {
        "PeriodMs": (double, True),
    }


class CollectionScheme(AWSProperty):
    """
    `CollectionScheme <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-collectionscheme.html>`__
    """

    props: PropsDictType = {
        "ConditionBasedCollectionScheme": (ConditionBasedCollectionScheme, False),
        "TimeBasedCollectionScheme": (TimeBasedCollectionScheme, False),
    }


class S3Config(AWSProperty):
    """
    `S3Config <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-s3config.html>`__
    """

    props: PropsDictType = {
        "BucketArn": (str, True),
        "DataFormat": (str, False),
        "Prefix": (str, False),
        "StorageCompressionFormat": (str, False),
    }


class TimestreamConfig(AWSProperty):
    """
    `TimestreamConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-timestreamconfig.html>`__
    """

    props: PropsDictType = {
        "ExecutionRoleArn": (str, True),
        "TimestreamTableArn": (str, True),
    }


class DataDestinationConfig(AWSProperty):
    """
    `DataDestinationConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-datadestinationconfig.html>`__
    """

    props: PropsDictType = {
        "S3Config": (S3Config, False),
        "TimestreamConfig": (TimestreamConfig, False),
    }


class SignalInformation(AWSProperty):
    """
    `SignalInformation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-campaign-signalinformation.html>`__
    """

    props: PropsDictType = {
        "MaxSampleCount": (double, False),
        "MinimumSamplingIntervalMs": (double, False),
        "Name": (str, True),
    }


class Campaign(AWSObject):
    """
    `Campaign <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotfleetwise-campaign.html>`__
    """

    resource_type = "AWS::IoTFleetWise::Campaign"

    props: PropsDictType = {
        "Action": (str, True),
        "CollectionScheme": (CollectionScheme, True),
        "Compression": (str, False),
        "DataDestinationConfigs": ([DataDestinationConfig], False),
        "DataExtraDimensions": ([str], False),
        "Description": (str, False),
        "DiagnosticsMode": (str, False),
        "ExpiryTime": (str, False),
        "Name": (str, True),
        "PostTriggerCollectionDuration": (double, False),
        "Priority": (integer, False),
        "SignalCatalogArn": (str, True),
        "SignalsToCollect": ([SignalInformation], False),
        "SpoolingMode": (str, False),
        "StartTime": (str, False),
        "Tags": (Tags, False),
        "TargetArn": (str, True),
    }


class CanInterface(AWSProperty):
    """
    `CanInterface <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-decodermanifest-caninterface.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "ProtocolName": (str, False),
        "ProtocolVersion": (str, False),
    }


class ObdInterface(AWSProperty):
    """
    `ObdInterface <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-decodermanifest-obdinterface.html>`__
    """

    props: PropsDictType = {
        "DtcRequestIntervalSeconds": (str, False),
        "HasTransmissionEcu": (str, False),
        "Name": (str, True),
        "ObdStandard": (str, False),
        "PidRequestIntervalSeconds": (str, False),
        "RequestMessageId": (str, True),
        "UseExtendedIds": (str, False),
    }


class NetworkInterfacesItems(AWSProperty):
    """
    `NetworkInterfacesItems <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-decodermanifest-networkinterfacesitems.html>`__
    """

    props: PropsDictType = {
        "CanInterface": (CanInterface, False),
        "InterfaceId": (str, True),
        "ObdInterface": (ObdInterface, False),
        "Type": (str, True),
    }


class CanSignal(AWSProperty):
    """
    `CanSignal <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-decodermanifest-cansignal.html>`__
    """

    props: PropsDictType = {
        "Factor": (str, True),
        "IsBigEndian": (str, True),
        "IsSigned": (str, True),
        "Length": (str, True),
        "MessageId": (str, True),
        "Name": (str, False),
        "Offset": (str, True),
        "StartBit": (str, True),
    }


class ObdSignal(AWSProperty):
    """
    `ObdSignal <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-decodermanifest-obdsignal.html>`__
    """

    props: PropsDictType = {
        "BitMaskLength": (str, False),
        "BitRightShift": (str, False),
        "ByteLength": (str, True),
        "Offset": (str, True),
        "Pid": (str, True),
        "PidResponseLength": (str, True),
        "Scaling": (str, True),
        "ServiceMode": (str, True),
        "StartByte": (str, True),
    }


class SignalDecodersItems(AWSProperty):
    """
    `SignalDecodersItems <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-decodermanifest-signaldecodersitems.html>`__
    """

    props: PropsDictType = {
        "CanSignal": (CanSignal, False),
        "FullyQualifiedName": (str, True),
        "InterfaceId": (str, True),
        "ObdSignal": (ObdSignal, False),
        "Type": (str, True),
    }


class DecoderManifest(AWSObject):
    """
    `DecoderManifest <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotfleetwise-decodermanifest.html>`__
    """

    resource_type = "AWS::IoTFleetWise::DecoderManifest"

    props: PropsDictType = {
        "Description": (str, False),
        "ModelManifestArn": (str, True),
        "Name": (str, True),
        "NetworkInterfaces": ([NetworkInterfacesItems], False),
        "SignalDecoders": ([SignalDecodersItems], False),
        "Status": (str, False),
        "Tags": (Tags, False),
    }


class Fleet(AWSObject):
    """
    `Fleet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotfleetwise-fleet.html>`__
    """

    resource_type = "AWS::IoTFleetWise::Fleet"

    props: PropsDictType = {
        "Description": (str, False),
        "Id": (str, True),
        "SignalCatalogArn": (str, True),
        "Tags": (Tags, False),
    }


class ModelManifest(AWSObject):
    """
    `ModelManifest <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotfleetwise-modelmanifest.html>`__
    """

    resource_type = "AWS::IoTFleetWise::ModelManifest"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Nodes": ([str], False),
        "SignalCatalogArn": (str, True),
        "Status": (str, False),
        "Tags": (Tags, False),
    }


class Actuator(AWSProperty):
    """
    `Actuator <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-signalcatalog-actuator.html>`__
    """

    props: PropsDictType = {
        "AllowedValues": ([str], False),
        "AssignedValue": (str, False),
        "DataType": (str, True),
        "Description": (str, False),
        "FullyQualifiedName": (str, True),
        "Max": (double, False),
        "Min": (double, False),
        "Unit": (str, False),
    }


class Attribute(AWSProperty):
    """
    `Attribute <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-signalcatalog-attribute.html>`__
    """

    props: PropsDictType = {
        "AllowedValues": ([str], False),
        "AssignedValue": (str, False),
        "DataType": (str, True),
        "DefaultValue": (str, False),
        "Description": (str, False),
        "FullyQualifiedName": (str, True),
        "Max": (double, False),
        "Min": (double, False),
        "Unit": (str, False),
    }


class Branch(AWSProperty):
    """
    `Branch <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-signalcatalog-branch.html>`__
    """

    props: PropsDictType = {
        "Description": (str, False),
        "FullyQualifiedName": (str, True),
    }


class Sensor(AWSProperty):
    """
    `Sensor <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-signalcatalog-sensor.html>`__
    """

    props: PropsDictType = {
        "AllowedValues": ([str], False),
        "DataType": (str, True),
        "Description": (str, False),
        "FullyQualifiedName": (str, True),
        "Max": (double, False),
        "Min": (double, False),
        "Unit": (str, False),
    }


class Node(AWSProperty):
    """
    `Node <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-signalcatalog-node.html>`__
    """

    props: PropsDictType = {
        "Actuator": (Actuator, False),
        "Attribute": (Attribute, False),
        "Branch": (Branch, False),
        "Sensor": (Sensor, False),
    }


class NodeCounts(AWSProperty):
    """
    `NodeCounts <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotfleetwise-signalcatalog-nodecounts.html>`__
    """

    props: PropsDictType = {
        "TotalActuators": (double, False),
        "TotalAttributes": (double, False),
        "TotalBranches": (double, False),
        "TotalNodes": (double, False),
        "TotalSensors": (double, False),
    }


class SignalCatalog(AWSObject):
    """
    `SignalCatalog <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotfleetwise-signalcatalog.html>`__
    """

    resource_type = "AWS::IoTFleetWise::SignalCatalog"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, False),
        "NodeCounts": (NodeCounts, False),
        "Nodes": ([Node], False),
        "Tags": (Tags, False),
    }


class Vehicle(AWSObject):
    """
    `Vehicle <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotfleetwise-vehicle.html>`__
    """

    resource_type = "AWS::IoTFleetWise::Vehicle"

    props: PropsDictType = {
        "AssociationBehavior": (str, False),
        "Attributes": (dict, False),
        "DecoderManifestArn": (str, True),
        "ModelManifestArn": (str, True),
        "Name": (str, True),
        "Tags": (Tags, False),
    }



from troposphere import Join, Ref, Template
from troposphere.cloudformation import AWSCustomObject


class CustomPlacementGroup(AWSCustomObject):
    resource_type = "Custom::PlacementGroup"

    props = {"ServiceToken": (str, True), "PlacementGroupName": (str, True)}


t = Template()

t.set_description(
    "Example template showing how a Lambda Function CustomResource might look"
    "For information on AWS Lambda-backed Custom Resources see:"
    "http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/"
    "template-custom-resources-lambda.html"
)

placementgroup_a = t.add_resource(
    CustomPlacementGroup(
        "ClusterGroup",
        ServiceToken=Join(
            "",
            [
                "arn:aws:lambda:",
                Ref("AWS::Region"),
                ":",
                Ref("AWS::AccountId"),
                ":function:cfnPlacementGroup",
            ],
        ),
        PlacementGroupName="ExampleClusterGroup",
    )
)

print(t.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import exactly_one, json_checker


def dict_or_string(x):
    """
    Property: Dashboard.DashboardBody
    """

    if isinstance(x, (dict, str)):
        return x
    raise TypeError(f"Value {x} of type {type(x)} must be either dict or str")


def validate_unit(unit):
    """
    Validate Units
    Property: MetricStat.Unit
    """

    VALID_UNITS = (
        "Seconds",
        "Microseconds",
        "Milliseconds",
        "Bytes",
        "Kilobytes",
        "Megabytes",
        "Gigabytes",
        "Terabytes",
        "Bits",
        "Kilobits",
        "Megabits",
        "Gigabits",
        "Terabits",
        "Percent",
        "Count",
        "Bytes/Second",
        "Kilobytes/Second",
        "Megabytes/Second",
        "Gigabytes/Second",
        "Terabytes/Second",
        "Bits/Second",
        "Kilobits/Second",
        "Megabits/Second",
        "Gigabits/Second",
        "Terabits/Second",
        "Count/Second",
        "None",
    )

    if unit not in VALID_UNITS:
        raise ValueError("MetricStat Unit must be one of: %s" % ", ".join(VALID_UNITS))
    return unit


def validate_treat_missing_data(value):
    """
    Validate TreatMissingData
    Property: Alarm.TreatMissingData
    """

    VALID_TREAT_MISSING_DATA_TYPES = ("breaching", "notBreaching", "ignore", "missing")

    if value not in VALID_TREAT_MISSING_DATA_TYPES:
        raise ValueError(
            "Alarm TreatMissingData must be one of: %s"
            % ", ".join(VALID_TREAT_MISSING_DATA_TYPES)
        )
    return value


def validate_alarm(self):
    """
    Class: Alarm
    """

    conds = [
        "ExtendedStatistic",
        "Metrics",
        "Statistic",
    ]
    exactly_one(self.__class__.__name__, self.properties, conds)


def validate_dashboard(self):
    """
    Class: Dashboard
    """

    name = "DashboardBody"
    if name in self.properties:
        dashboard_body = self.properties.get(name)
        self.properties[name] = json_checker(dashboard_body)



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import integer


class ExperimentTemplateAction(AWSProperty):
    """
    `ExperimentTemplateAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-experimenttemplateaction.html>`__
    """

    props: PropsDictType = {
        "ActionId": (str, True),
        "Description": (str, False),
        "Parameters": (dict, False),
        "StartAfter": ([str], False),
        "Targets": (dict, False),
    }


class ExperimentTemplateExperimentOptions(AWSProperty):
    """
    `ExperimentTemplateExperimentOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-experimenttemplateexperimentoptions.html>`__
    """

    props: PropsDictType = {
        "AccountTargeting": (str, False),
        "EmptyTargetResolutionMode": (str, False),
    }


class CloudWatchLogsConfiguration(AWSProperty):
    """
    `CloudWatchLogsConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-cloudwatchlogsconfiguration.html>`__
    """

    props: PropsDictType = {
        "LogGroupArn": (str, True),
    }


class S3Configuration(AWSProperty):
    """
    `S3Configuration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-s3configuration.html>`__
    """

    props: PropsDictType = {
        "BucketName": (str, True),
        "Prefix": (str, False),
    }


class ExperimentTemplateLogConfiguration(AWSProperty):
    """
    `ExperimentTemplateLogConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-experimenttemplatelogconfiguration.html>`__
    """

    props: PropsDictType = {
        "CloudWatchLogsConfiguration": (CloudWatchLogsConfiguration, False),
        "LogSchemaVersion": (integer, True),
        "S3Configuration": (S3Configuration, False),
    }


class ExperimentTemplateStopCondition(AWSProperty):
    """
    `ExperimentTemplateStopCondition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-experimenttemplatestopcondition.html>`__
    """

    props: PropsDictType = {
        "Source": (str, True),
        "Value": (str, False),
    }


class ExperimentTemplateTargetFilter(AWSProperty):
    """
    `ExperimentTemplateTargetFilter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-experimenttemplatetargetfilter.html>`__
    """

    props: PropsDictType = {
        "Path": (str, True),
        "Values": ([str], True),
    }


class ExperimentTemplateTarget(AWSProperty):
    """
    `ExperimentTemplateTarget <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-fis-experimenttemplate-experimenttemplatetarget.html>`__
    """

    props: PropsDictType = {
        "Filters": ([ExperimentTemplateTargetFilter], False),
        "Parameters": (dict, False),
        "ResourceArns": ([str], False),
        "ResourceTags": (dict, False),
        "ResourceType": (str, True),
        "SelectionMode": (str, True),
    }


class ExperimentTemplate(AWSObject):
    """
    `ExperimentTemplate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-fis-experimenttemplate.html>`__
    """

    resource_type = "AWS::FIS::ExperimentTemplate"

    props: PropsDictType = {
        "Actions": (dict, False),
        "Description": (str, True),
        "ExperimentOptions": (ExperimentTemplateExperimentOptions, False),
        "LogConfiguration": (ExperimentTemplateLogConfiguration, False),
        "RoleArn": (str, True),
        "StopConditions": ([ExperimentTemplateStopCondition], True),
        "Tags": (dict, True),
        "Targets": (dict, True),
    }


class TargetAccountConfiguration(AWSObject):
    """
    `TargetAccountConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-fis-targetaccountconfiguration.html>`__
    """

    resource_type = "AWS::FIS::TargetAccountConfiguration"

    props: PropsDictType = {
        "AccountId": (str, True),
        "Description": (str, False),
        "ExperimentTemplateId": (str, True),
        "RoleArn": (str, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer
from .validators.networkfirewall import validate_rule_group_type


class SubnetMapping(AWSProperty):
    """
    `SubnetMapping <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewall-subnetmapping.html>`__
    """

    props: PropsDictType = {
        "IPAddressType": (str, False),
        "SubnetId": (str, True),
    }


class Firewall(AWSObject):
    """
    `Firewall <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkfirewall-firewall.html>`__
    """

    resource_type = "AWS::NetworkFirewall::Firewall"

    props: PropsDictType = {
        "DeleteProtection": (boolean, False),
        "Description": (str, False),
        "FirewallName": (str, True),
        "FirewallPolicyArn": (str, True),
        "FirewallPolicyChangeProtection": (boolean, False),
        "SubnetChangeProtection": (boolean, False),
        "SubnetMappings": ([SubnetMapping], True),
        "Tags": (Tags, False),
        "VpcId": (str, True),
    }


class Dimension(AWSProperty):
    """
    `Dimension <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-dimension.html>`__
    """

    props: PropsDictType = {
        "Value": (str, True),
    }


class PublishMetricAction(AWSProperty):
    """
    `PublishMetricAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-publishmetricaction.html>`__
    """

    props: PropsDictType = {
        "Dimensions": ([Dimension], True),
    }


class ActionDefinition(AWSProperty):
    """
    `ActionDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-actiondefinition.html>`__
    """

    props: PropsDictType = {
        "PublishMetricAction": (PublishMetricAction, False),
    }


class CustomAction(AWSProperty):
    """
    `CustomAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-customaction.html>`__
    """

    props: PropsDictType = {
        "ActionDefinition": (ActionDefinition, True),
        "ActionName": (str, True),
    }


class IPSet(AWSProperty):
    """
    `IPSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-ipset.html>`__
    """

    props: PropsDictType = {
        "Definition": ([str], False),
    }


class PolicyVariables(AWSProperty):
    """
    `PolicyVariables <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewallpolicy-policyvariables.html>`__
    """

    props: PropsDictType = {
        "RuleVariables": (dict, False),
    }


class StatefulEngineOptions(AWSProperty):
    """
    `StatefulEngineOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewallpolicy-statefulengineoptions.html>`__
    """

    props: PropsDictType = {
        "RuleOrder": (str, False),
        "StreamExceptionPolicy": (str, False),
    }


class StatefulRuleGroupOverride(AWSProperty):
    """
    `StatefulRuleGroupOverride <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewallpolicy-statefulrulegroupoverride.html>`__
    """

    props: PropsDictType = {
        "Action": (str, False),
    }


class StatefulRuleGroupReference(AWSProperty):
    """
    `StatefulRuleGroupReference <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewallpolicy-statefulrulegroupreference.html>`__
    """

    props: PropsDictType = {
        "Override": (StatefulRuleGroupOverride, False),
        "Priority": (integer, False),
        "ResourceArn": (str, True),
    }


class StatelessRuleGroupReference(AWSProperty):
    """
    `StatelessRuleGroupReference <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewallpolicy-statelessrulegroupreference.html>`__
    """

    props: PropsDictType = {
        "Priority": (integer, True),
        "ResourceArn": (str, True),
    }


class FirewallPolicyProperty(AWSProperty):
    """
    `FirewallPolicyProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-firewallpolicy-firewallpolicy.html>`__
    """

    props: PropsDictType = {
        "PolicyVariables": (PolicyVariables, False),
        "StatefulDefaultActions": ([str], False),
        "StatefulEngineOptions": (StatefulEngineOptions, False),
        "StatefulRuleGroupReferences": ([StatefulRuleGroupReference], False),
        "StatelessCustomActions": ([CustomAction], False),
        "StatelessDefaultActions": ([str], True),
        "StatelessFragmentDefaultActions": ([str], True),
        "StatelessRuleGroupReferences": ([StatelessRuleGroupReference], False),
        "TLSInspectionConfigurationArn": (str, False),
    }


class FirewallPolicy(AWSObject):
    """
    `FirewallPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkfirewall-firewallpolicy.html>`__
    """

    resource_type = "AWS::NetworkFirewall::FirewallPolicy"

    props: PropsDictType = {
        "Description": (str, False),
        "FirewallPolicy": (FirewallPolicyProperty, True),
        "FirewallPolicyName": (str, True),
        "Tags": (Tags, False),
    }


class LogDestinationConfig(AWSProperty):
    """
    `LogDestinationConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-loggingconfiguration-logdestinationconfig.html>`__
    """

    props: PropsDictType = {
        "LogDestination": (dict, True),
        "LogDestinationType": (str, True),
        "LogType": (str, True),
    }


class LoggingConfigurationProperty(AWSProperty):
    """
    `LoggingConfigurationProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-loggingconfiguration-loggingconfiguration.html>`__
    """

    props: PropsDictType = {
        "LogDestinationConfigs": ([LogDestinationConfig], True),
    }


class LoggingConfiguration(AWSObject):
    """
    `LoggingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkfirewall-loggingconfiguration.html>`__
    """

    resource_type = "AWS::NetworkFirewall::LoggingConfiguration"

    props: PropsDictType = {
        "FirewallArn": (str, True),
        "FirewallName": (str, False),
        "LoggingConfiguration": (LoggingConfigurationProperty, True),
    }


class IPSetReference(AWSProperty):
    """
    `IPSetReference <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-ipsetreference.html>`__
    """

    props: PropsDictType = {
        "ReferenceArn": (str, False),
    }


class ReferenceSets(AWSProperty):
    """
    `ReferenceSets <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-referencesets.html>`__
    """

    props: PropsDictType = {
        "IPSetReferences": (dict, False),
    }


class PortSet(AWSProperty):
    """
    `PortSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-portset.html>`__
    """

    props: PropsDictType = {
        "Definition": ([str], False),
    }


class RuleVariables(AWSProperty):
    """
    `RuleVariables <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-rulevariables.html>`__
    """

    props: PropsDictType = {
        "IPSets": (dict, False),
        "PortSets": (dict, False),
    }


class RulesSourceList(AWSProperty):
    """
    `RulesSourceList <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-rulessourcelist.html>`__
    """

    props: PropsDictType = {
        "GeneratedRulesType": (str, True),
        "TargetTypes": ([str], True),
        "Targets": ([str], True),
    }


class Header(AWSProperty):
    """
    `Header <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-header.html>`__
    """

    props: PropsDictType = {
        "Destination": (str, True),
        "DestinationPort": (str, True),
        "Direction": (str, True),
        "Protocol": (str, True),
        "Source": (str, True),
        "SourcePort": (str, True),
    }


class RuleOption(AWSProperty):
    """
    `RuleOption <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-ruleoption.html>`__
    """

    props: PropsDictType = {
        "Keyword": (str, True),
        "Settings": ([str], False),
    }


class StatefulRule(AWSProperty):
    """
    `StatefulRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-statefulrule.html>`__
    """

    props: PropsDictType = {
        "Action": (str, True),
        "Header": (Header, True),
        "RuleOptions": ([RuleOption], True),
    }


class Address(AWSProperty):
    """
    `Address <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-address.html>`__
    """

    props: PropsDictType = {
        "AddressDefinition": (str, True),
    }


class PortRange(AWSProperty):
    """
    `PortRange <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-portrange.html>`__
    """

    props: PropsDictType = {
        "FromPort": (integer, True),
        "ToPort": (integer, True),
    }


class TCPFlagField(AWSProperty):
    """
    `TCPFlagField <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-tcpflagfield.html>`__
    """

    props: PropsDictType = {
        "Flags": ([str], True),
        "Masks": ([str], False),
    }


class MatchAttributes(AWSProperty):
    """
    `MatchAttributes <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-matchattributes.html>`__
    """

    props: PropsDictType = {
        "DestinationPorts": ([PortRange], False),
        "Destinations": ([Address], False),
        "Protocols": ([integer], False),
        "SourcePorts": ([PortRange], False),
        "Sources": ([Address], False),
        "TCPFlags": ([TCPFlagField], False),
    }


class RuleDefinition(AWSProperty):
    """
    `RuleDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-ruledefinition.html>`__
    """

    props: PropsDictType = {
        "Actions": ([str], True),
        "MatchAttributes": (MatchAttributes, True),
    }


class StatelessRule(AWSProperty):
    """
    `StatelessRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-statelessrule.html>`__
    """

    props: PropsDictType = {
        "Priority": (integer, True),
        "RuleDefinition": (RuleDefinition, True),
    }


class StatelessRulesAndCustomActions(AWSProperty):
    """
    `StatelessRulesAndCustomActions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-statelessrulesandcustomactions.html>`__
    """

    props: PropsDictType = {
        "CustomActions": ([CustomAction], False),
        "StatelessRules": ([StatelessRule], True),
    }


class RulesSource(AWSProperty):
    """
    `RulesSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-rulessource.html>`__
    """

    props: PropsDictType = {
        "RulesSourceList": (RulesSourceList, False),
        "RulesString": (str, False),
        "StatefulRules": ([StatefulRule], False),
        "StatelessRulesAndCustomActions": (StatelessRulesAndCustomActions, False),
    }


class StatefulRuleOptions(AWSProperty):
    """
    `StatefulRuleOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-statefulruleoptions.html>`__
    """

    props: PropsDictType = {
        "RuleOrder": (str, False),
    }


class RuleGroupProperty(AWSProperty):
    """
    `RuleGroupProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-rulegroup-rulegroup.html>`__
    """

    props: PropsDictType = {
        "ReferenceSets": (ReferenceSets, False),
        "RuleVariables": (RuleVariables, False),
        "RulesSource": (RulesSource, True),
        "StatefulRuleOptions": (StatefulRuleOptions, False),
    }


class RuleGroup(AWSObject):
    """
    `RuleGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkfirewall-rulegroup.html>`__
    """

    resource_type = "AWS::NetworkFirewall::RuleGroup"

    props: PropsDictType = {
        "Capacity": (integer, True),
        "Description": (str, False),
        "RuleGroup": (RuleGroupProperty, False),
        "RuleGroupName": (str, True),
        "Tags": (Tags, False),
        "Type": (validate_rule_group_type, True),
    }


class CheckCertificateRevocationStatus(AWSProperty):
    """
    `CheckCertificateRevocationStatus <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-checkcertificaterevocationstatus.html>`__
    """

    props: PropsDictType = {
        "RevokedStatusAction": (str, False),
        "UnknownStatusAction": (str, False),
    }


class ServerCertificate(AWSProperty):
    """
    `ServerCertificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-servercertificate.html>`__
    """

    props: PropsDictType = {
        "ResourceArn": (str, False),
    }


class ServerCertificateScope(AWSProperty):
    """
    `ServerCertificateScope <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-servercertificatescope.html>`__
    """

    props: PropsDictType = {
        "DestinationPorts": ([PortRange], False),
        "Destinations": ([Address], False),
        "Protocols": ([integer], False),
        "SourcePorts": ([PortRange], False),
        "Sources": ([Address], False),
    }


class ServerCertificateConfiguration(AWSProperty):
    """
    `ServerCertificateConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-servercertificateconfiguration.html>`__
    """

    props: PropsDictType = {
        "CertificateAuthorityArn": (str, False),
        "CheckCertificateRevocationStatus": (CheckCertificateRevocationStatus, False),
        "Scopes": ([ServerCertificateScope], False),
        "ServerCertificates": ([ServerCertificate], False),
    }


class TLSInspectionConfigurationProperty(AWSProperty):
    """
    `TLSInspectionConfigurationProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkfirewall-tlsinspectionconfiguration-tlsinspectionconfiguration.html>`__
    """

    props: PropsDictType = {
        "ServerCertificateConfigurations": ([ServerCertificateConfiguration], False),
    }


class TLSInspectionConfiguration(AWSObject):
    """
    `TLSInspectionConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkfirewall-tlsinspectionconfiguration.html>`__
    """

    resource_type = "AWS::NetworkFirewall::TLSInspectionConfiguration"

    props: PropsDictType = {
        "Description": (str, False),
        "TLSInspectionConfiguration": (TLSInspectionConfigurationProperty, True),
        "TLSInspectionConfigurationName": (str, True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import integer


class CapacityUnitsConfiguration(AWSProperty):
    """
    `CapacityUnitsConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kendraranking-executionplan-capacityunitsconfiguration.html>`__
    """

    props: PropsDictType = {
        "RescoreCapacityUnits": (integer, True),
    }


class ExecutionPlan(AWSObject):
    """
    `ExecutionPlan <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kendraranking-executionplan.html>`__
    """

    resource_type = "AWS::KendraRanking::ExecutionPlan"

    props: PropsDictType = {
        "CapacityUnits": (CapacityUnitsConfiguration, False),
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }



import unittest

from troposphere import Ref
from troposphere.cloudformation import WaitCondition, WaitConditionHandle
from troposphere.policies import CreationPolicy, ResourceSignal


class TestWaitCondition(unittest.TestCase):
    def test_CreationPolicy(self):
        w = WaitCondition(
            "mycondition",
            CreationPolicy=CreationPolicy(
                ResourceSignal=ResourceSignal(Timeout="PT15M")
            ),
        )
        w.validate()

    def test_CreationPolicyWithProps(self):
        w = WaitCondition(
            "mycondition",
            Count=10,
            CreationPolicy=CreationPolicy(
                ResourceSignal=ResourceSignal(Timeout="PT15M")
            ),
        )
        with self.assertRaises(ValueError):
            w.validate()

    def test_RequiredProps(self):
        handle = WaitConditionHandle("myWaitHandle")
        w = WaitCondition(
            "mycondition",
            Handle=Ref(handle),
            Timeout="300",
        )
        w.validate()


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import integer
from .validators.resiliencehub import (
    validate_resiliencypolicy_policy,
    validate_resiliencypolicy_tier,
)


class EventSubscription(AWSProperty):
    """
    `EventSubscription <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resiliencehub-app-eventsubscription.html>`__
    """

    props: PropsDictType = {
        "EventType": (str, True),
        "Name": (str, True),
        "SnsTopicArn": (str, False),
    }


class PermissionModel(AWSProperty):
    """
    `PermissionModel <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resiliencehub-app-permissionmodel.html>`__
    """

    props: PropsDictType = {
        "CrossAccountRoleArns": ([str], False),
        "InvokerRoleName": (str, False),
        "Type": (str, True),
    }


class PhysicalResourceId(AWSProperty):
    """
    `PhysicalResourceId <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resiliencehub-app-physicalresourceid.html>`__
    """

    props: PropsDictType = {
        "AwsAccountId": (str, False),
        "AwsRegion": (str, False),
        "Identifier": (str, True),
        "Type": (str, True),
    }


class ResourceMapping(AWSProperty):
    """
    `ResourceMapping <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resiliencehub-app-resourcemapping.html>`__
    """

    props: PropsDictType = {
        "EksSourceName": (str, False),
        "LogicalStackName": (str, False),
        "MappingType": (str, True),
        "PhysicalResourceId": (PhysicalResourceId, True),
        "ResourceName": (str, False),
        "TerraformSourceName": (str, False),
    }


class App(AWSObject):
    """
    `App <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-resiliencehub-app.html>`__
    """

    resource_type = "AWS::ResilienceHub::App"

    props: PropsDictType = {
        "AppAssessmentSchedule": (str, False),
        "AppTemplateBody": (str, True),
        "Description": (str, False),
        "EventSubscriptions": ([EventSubscription], False),
        "Name": (str, True),
        "PermissionModel": (PermissionModel, False),
        "ResiliencyPolicyArn": (str, False),
        "ResourceMappings": ([ResourceMapping], True),
        "Tags": (dict, False),
    }


class FailurePolicy(AWSProperty):
    """
    `FailurePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resiliencehub-resiliencypolicy-failurepolicy.html>`__
    """

    props: PropsDictType = {
        "RpoInSecs": (integer, True),
        "RtoInSecs": (integer, True),
    }


class ResiliencyPolicy(AWSObject):
    """
    `ResiliencyPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-resiliencehub-resiliencypolicy.html>`__
    """

    resource_type = "AWS::ResilienceHub::ResiliencyPolicy"

    props: PropsDictType = {
        "DataLocationConstraint": (str, False),
        "Policy": (validate_resiliencypolicy_policy, True),
        "PolicyDescription": (str, False),
        "PolicyName": (str, True),
        "Tags": (dict, False),
        "Tier": (validate_resiliencypolicy_tier, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer
from .validators.servicecatalog import validate_tag_update


class AcceptedPortfolioShare(AWSObject):
    """
    `AcceptedPortfolioShare <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-acceptedportfolioshare.html>`__
    """

    resource_type = "AWS::ServiceCatalog::AcceptedPortfolioShare"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "PortfolioId": (str, True),
    }


class ProvisioningArtifactProperties(AWSProperty):
    """
    `ProvisioningArtifactProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-cloudformationproduct-provisioningartifactproperties.html>`__
    """

    props: PropsDictType = {
        "Description": (str, False),
        "DisableTemplateValidation": (boolean, False),
        "Info": (dict, True),
        "Name": (str, False),
        "Type": (str, False),
    }


class CodeStarParameters(AWSProperty):
    """
    `CodeStarParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-cloudformationproduct-codestarparameters.html>`__
    """

    props: PropsDictType = {
        "ArtifactPath": (str, True),
        "Branch": (str, True),
        "ConnectionArn": (str, True),
        "Repository": (str, True),
    }


class ConnectionParameters(AWSProperty):
    """
    `ConnectionParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-cloudformationproduct-sourceconnection-connectionparameters.html>`__
    """

    props: PropsDictType = {
        "CodeStar": (CodeStarParameters, False),
    }


class SourceConnection(AWSProperty):
    """
    `SourceConnection <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-cloudformationproduct-sourceconnection.html>`__
    """

    props: PropsDictType = {
        "ConnectionParameters": (ConnectionParameters, True),
        "Type": (str, True),
    }


class CloudFormationProduct(AWSObject):
    """
    `CloudFormationProduct <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-cloudformationproduct.html>`__
    """

    resource_type = "AWS::ServiceCatalog::CloudFormationProduct"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Description": (str, False),
        "Distributor": (str, False),
        "Name": (str, True),
        "Owner": (str, True),
        "ProductType": (str, False),
        "ProvisioningArtifactParameters": ([ProvisioningArtifactProperties], False),
        "ReplaceProvisioningArtifacts": (boolean, False),
        "SourceConnection": (SourceConnection, False),
        "SupportDescription": (str, False),
        "SupportEmail": (str, False),
        "SupportUrl": (str, False),
        "Tags": (Tags, False),
    }


class ProvisioningParameter(AWSProperty):
    """
    `ProvisioningParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-cloudformationprovisionedproduct-provisioningparameter.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class ProvisioningPreferences(AWSProperty):
    """
    `ProvisioningPreferences <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-cloudformationprovisionedproduct-provisioningpreferences.html>`__
    """

    props: PropsDictType = {
        "StackSetAccounts": ([str], False),
        "StackSetFailureToleranceCount": (integer, False),
        "StackSetFailureTolerancePercentage": (integer, False),
        "StackSetMaxConcurrencyCount": (integer, False),
        "StackSetMaxConcurrencyPercentage": (integer, False),
        "StackSetOperationType": (str, False),
        "StackSetRegions": ([str], False),
    }


class CloudFormationProvisionedProduct(AWSObject):
    """
    `CloudFormationProvisionedProduct <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-cloudformationprovisionedproduct.html>`__
    """

    resource_type = "AWS::ServiceCatalog::CloudFormationProvisionedProduct"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "NotificationArns": ([str], False),
        "PathId": (str, False),
        "PathName": (str, False),
        "ProductId": (str, False),
        "ProductName": (str, False),
        "ProvisionedProductName": (str, False),
        "ProvisioningArtifactId": (str, False),
        "ProvisioningArtifactName": (str, False),
        "ProvisioningParameters": ([ProvisioningParameter], False),
        "ProvisioningPreferences": (ProvisioningPreferences, False),
        "Tags": (Tags, False),
    }


class LaunchNotificationConstraint(AWSObject):
    """
    `LaunchNotificationConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-launchnotificationconstraint.html>`__
    """

    resource_type = "AWS::ServiceCatalog::LaunchNotificationConstraint"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Description": (str, False),
        "NotificationArns": ([str], True),
        "PortfolioId": (str, True),
        "ProductId": (str, True),
    }


class LaunchRoleConstraint(AWSObject):
    """
    `LaunchRoleConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-launchroleconstraint.html>`__
    """

    resource_type = "AWS::ServiceCatalog::LaunchRoleConstraint"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Description": (str, False),
        "LocalRoleName": (str, False),
        "PortfolioId": (str, True),
        "ProductId": (str, True),
        "RoleArn": (str, False),
    }


class LaunchTemplateConstraint(AWSObject):
    """
    `LaunchTemplateConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-launchtemplateconstraint.html>`__
    """

    resource_type = "AWS::ServiceCatalog::LaunchTemplateConstraint"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Description": (str, False),
        "PortfolioId": (str, True),
        "ProductId": (str, True),
        "Rules": (str, True),
    }


class Portfolio(AWSObject):
    """
    `Portfolio <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolio.html>`__
    """

    resource_type = "AWS::ServiceCatalog::Portfolio"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Description": (str, False),
        "DisplayName": (str, True),
        "ProviderName": (str, True),
        "Tags": (Tags, False),
    }


class PortfolioPrincipalAssociation(AWSObject):
    """
    `PortfolioPrincipalAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolioprincipalassociation.html>`__
    """

    resource_type = "AWS::ServiceCatalog::PortfolioPrincipalAssociation"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "PortfolioId": (str, True),
        "PrincipalARN": (str, True),
        "PrincipalType": (str, True),
    }


class PortfolioProductAssociation(AWSObject):
    """
    `PortfolioProductAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolioproductassociation.html>`__
    """

    resource_type = "AWS::ServiceCatalog::PortfolioProductAssociation"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "PortfolioId": (str, True),
        "ProductId": (str, True),
        "SourcePortfolioId": (str, False),
    }


class PortfolioShare(AWSObject):
    """
    `PortfolioShare <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-portfolioshare.html>`__
    """

    resource_type = "AWS::ServiceCatalog::PortfolioShare"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "AccountId": (str, True),
        "PortfolioId": (str, True),
        "ShareTagOptions": (boolean, False),
    }


class ResourceUpdateConstraint(AWSObject):
    """
    `ResourceUpdateConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-resourceupdateconstraint.html>`__
    """

    resource_type = "AWS::ServiceCatalog::ResourceUpdateConstraint"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Description": (str, False),
        "PortfolioId": (str, True),
        "ProductId": (str, True),
        "TagUpdateOnProvisionedProduct": (validate_tag_update, True),
    }


class DefinitionParameter(AWSProperty):
    """
    `DefinitionParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-servicecatalog-serviceaction-definitionparameter.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class ServiceAction(AWSObject):
    """
    `ServiceAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-serviceaction.html>`__
    """

    resource_type = "AWS::ServiceCatalog::ServiceAction"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "Definition": ([DefinitionParameter], True),
        "DefinitionType": (str, True),
        "Description": (str, False),
        "Name": (str, True),
    }


class ServiceActionAssociation(AWSObject):
    """
    `ServiceActionAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-serviceactionassociation.html>`__
    """

    resource_type = "AWS::ServiceCatalog::ServiceActionAssociation"

    props: PropsDictType = {
        "ProductId": (str, True),
        "ProvisioningArtifactId": (str, True),
        "ServiceActionId": (str, True),
    }


class StackSetConstraint(AWSObject):
    """
    `StackSetConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-stacksetconstraint.html>`__
    """

    resource_type = "AWS::ServiceCatalog::StackSetConstraint"

    props: PropsDictType = {
        "AcceptLanguage": (str, False),
        "AccountList": ([str], True),
        "AdminRole": (str, True),
        "Description": (str, True),
        "ExecutionRole": (str, True),
        "PortfolioId": (str, True),
        "ProductId": (str, True),
        "RegionList": ([str], True),
        "StackInstanceControl": (str, True),
    }


class TagOption(AWSObject):
    """
    `TagOption <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-tagoption.html>`__
    """

    resource_type = "AWS::ServiceCatalog::TagOption"

    props: PropsDictType = {
        "Active": (boolean, False),
        "Key": (str, True),
        "Value": (str, True),
    }


class TagOptionAssociation(AWSObject):
    """
    `TagOptionAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicecatalog-tagoptionassociation.html>`__
    """

    resource_type = "AWS::ServiceCatalog::TagOptionAssociation"

    props: PropsDictType = {
        "ResourceId": (str, True),
        "TagOptionId": (str, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer


class RoutingStrategy(AWSProperty):
    """
    `RoutingStrategy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-alias-routingstrategy.html>`__
    """

    props: PropsDictType = {
        "FleetId": (str, False),
        "Message": (str, False),
        "Type": (str, True),
    }


class Alias(AWSObject):
    """
    `Alias <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html>`__
    """

    resource_type = "AWS::GameLift::Alias"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "RoutingStrategy": (RoutingStrategy, True),
    }


class StorageLocation(AWSProperty):
    """
    `StorageLocation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-build-storagelocation.html>`__
    """

    props: PropsDictType = {
        "Bucket": (str, True),
        "Key": (str, True),
        "ObjectVersion": (str, False),
        "RoleArn": (str, True),
    }


class Build(AWSObject):
    """
    `Build <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html>`__
    """

    resource_type = "AWS::GameLift::Build"

    props: PropsDictType = {
        "Name": (str, False),
        "OperatingSystem": (str, False),
        "ServerSdkVersion": (str, False),
        "StorageLocation": (StorageLocation, False),
        "Version": (str, False),
    }


class AnywhereConfiguration(AWSProperty):
    """
    `AnywhereConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-anywhereconfiguration.html>`__
    """

    props: PropsDictType = {
        "Cost": (str, True),
    }


class CertificateConfiguration(AWSProperty):
    """
    `CertificateConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-certificateconfiguration.html>`__
    """

    props: PropsDictType = {
        "CertificateType": (str, True),
    }


class IpPermission(AWSProperty):
    """
    `IpPermission <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-ippermission.html>`__
    """

    props: PropsDictType = {
        "FromPort": (integer, True),
        "IpRange": (str, True),
        "Protocol": (str, True),
        "ToPort": (integer, True),
    }


class LocationCapacity(AWSProperty):
    """
    `LocationCapacity <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-locationcapacity.html>`__
    """

    props: PropsDictType = {
        "DesiredEC2Instances": (integer, True),
        "MaxSize": (integer, True),
        "MinSize": (integer, True),
    }


class LocationConfiguration(AWSProperty):
    """
    `LocationConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-locationconfiguration.html>`__
    """

    props: PropsDictType = {
        "Location": (str, True),
        "LocationCapacity": (LocationCapacity, False),
    }


class ResourceCreationLimitPolicy(AWSProperty):
    """
    `ResourceCreationLimitPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-resourcecreationlimitpolicy.html>`__
    """

    props: PropsDictType = {
        "NewGameSessionsPerCreator": (integer, False),
        "PolicyPeriodInMinutes": (integer, False),
    }


class ServerProcess(AWSProperty):
    """
    `ServerProcess <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-serverprocess.html>`__
    """

    props: PropsDictType = {
        "ConcurrentExecutions": (integer, True),
        "LaunchPath": (str, True),
        "Parameters": (str, False),
    }


class RuntimeConfiguration(AWSProperty):
    """
    `RuntimeConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-runtimeconfiguration.html>`__
    """

    props: PropsDictType = {
        "GameSessionActivationTimeoutSeconds": (integer, False),
        "MaxConcurrentGameSessionActivations": (integer, False),
        "ServerProcesses": ([ServerProcess], False),
    }


class TargetConfiguration(AWSProperty):
    """
    `TargetConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-targetconfiguration.html>`__
    """

    props: PropsDictType = {
        "TargetValue": (double, True),
    }


class ScalingPolicy(AWSProperty):
    """
    `ScalingPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-scalingpolicy.html>`__
    """

    props: PropsDictType = {
        "ComparisonOperator": (str, False),
        "EvaluationPeriods": (integer, False),
        "Location": (str, False),
        "MetricName": (str, True),
        "Name": (str, True),
        "PolicyType": (str, False),
        "ScalingAdjustment": (integer, False),
        "ScalingAdjustmentType": (str, False),
        "Status": (str, False),
        "TargetConfiguration": (TargetConfiguration, False),
        "Threshold": (double, False),
        "UpdateStatus": (str, False),
    }


class Fleet(AWSObject):
    """
    `Fleet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html>`__
    """

    resource_type = "AWS::GameLift::Fleet"

    props: PropsDictType = {
        "AnywhereConfiguration": (AnywhereConfiguration, False),
        "ApplyCapacity": (str, False),
        "BuildId": (str, False),
        "CertificateConfiguration": (CertificateConfiguration, False),
        "ComputeType": (str, False),
        "Description": (str, False),
        "DesiredEC2Instances": (integer, False),
        "EC2InboundPermissions": ([IpPermission], False),
        "EC2InstanceType": (str, False),
        "FleetType": (str, False),
        "InstanceRoleARN": (str, False),
        "InstanceRoleCredentialsProvider": (str, False),
        "Locations": ([LocationConfiguration], False),
        "MaxSize": (integer, False),
        "MetricGroups": ([str], False),
        "MinSize": (integer, False),
        "Name": (str, True),
        "NewGameSessionProtectionPolicy": (str, False),
        "PeerVpcAwsAccountId": (str, False),
        "PeerVpcId": (str, False),
        "ResourceCreationLimitPolicy": (ResourceCreationLimitPolicy, False),
        "RuntimeConfiguration": (RuntimeConfiguration, False),
        "ScalingPolicies": ([ScalingPolicy], False),
        "ScriptId": (str, False),
    }


class TargetTrackingConfiguration(AWSProperty):
    """
    `TargetTrackingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gameservergroup-targettrackingconfiguration.html>`__
    """

    props: PropsDictType = {
        "TargetValue": (double, True),
    }


class AutoScalingPolicy(AWSProperty):
    """
    `AutoScalingPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gameservergroup-autoscalingpolicy.html>`__
    """

    props: PropsDictType = {
        "EstimatedInstanceWarmup": (double, False),
        "TargetTrackingConfiguration": (TargetTrackingConfiguration, True),
    }


class InstanceDefinition(AWSProperty):
    """
    `InstanceDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gameservergroup-instancedefinition.html>`__
    """

    props: PropsDictType = {
        "InstanceType": (str, True),
        "WeightedCapacity": (str, False),
    }


class LaunchTemplate(AWSProperty):
    """
    `LaunchTemplate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gameservergroup-launchtemplate.html>`__
    """

    props: PropsDictType = {
        "LaunchTemplateId": (str, False),
        "LaunchTemplateName": (str, False),
        "Version": (str, False),
    }


class GameServerGroup(AWSObject):
    """
    `GameServerGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-gameservergroup.html>`__
    """

    resource_type = "AWS::GameLift::GameServerGroup"

    props: PropsDictType = {
        "AutoScalingPolicy": (AutoScalingPolicy, False),
        "BalancingStrategy": (str, False),
        "DeleteOption": (str, False),
        "GameServerGroupName": (str, True),
        "GameServerProtectionPolicy": (str, False),
        "InstanceDefinitions": ([InstanceDefinition], True),
        "LaunchTemplate": (LaunchTemplate, False),
        "MaxSize": (double, False),
        "MinSize": (double, False),
        "RoleArn": (str, True),
        "Tags": (Tags, False),
        "VpcSubnets": ([str], False),
    }


class FilterConfiguration(AWSProperty):
    """
    `FilterConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gamesessionqueue-filterconfiguration.html>`__
    """

    props: PropsDictType = {
        "AllowedLocations": ([str], False),
    }


class GameSessionQueueDestination(AWSProperty):
    """
    `GameSessionQueueDestination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gamesessionqueue-gamesessionqueuedestination.html>`__
    """

    props: PropsDictType = {
        "DestinationArn": (str, False),
    }


class PlayerLatencyPolicy(AWSProperty):
    """
    `PlayerLatencyPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gamesessionqueue-playerlatencypolicy.html>`__
    """

    props: PropsDictType = {
        "MaximumIndividualPlayerLatencyMilliseconds": (integer, False),
        "PolicyDurationSeconds": (integer, False),
    }


class PriorityConfiguration(AWSProperty):
    """
    `PriorityConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-gamesessionqueue-priorityconfiguration.html>`__
    """

    props: PropsDictType = {
        "LocationOrder": ([str], False),
        "PriorityOrder": ([str], False),
    }


class GameSessionQueue(AWSObject):
    """
    `GameSessionQueue <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-gamesessionqueue.html>`__
    """

    resource_type = "AWS::GameLift::GameSessionQueue"

    props: PropsDictType = {
        "CustomEventData": (str, False),
        "Destinations": ([GameSessionQueueDestination], False),
        "FilterConfiguration": (FilterConfiguration, False),
        "Name": (str, True),
        "NotificationTarget": (str, False),
        "PlayerLatencyPolicies": ([PlayerLatencyPolicy], False),
        "PriorityConfiguration": (PriorityConfiguration, False),
        "Tags": (Tags, False),
        "TimeoutInSeconds": (integer, False),
    }


class Location(AWSObject):
    """
    `Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-location.html>`__
    """

    resource_type = "AWS::GameLift::Location"

    props: PropsDictType = {
        "LocationName": (str, True),
        "Tags": (Tags, False),
    }


class GameProperty(AWSProperty):
    """
    `GameProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-matchmakingconfiguration-gameproperty.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class MatchmakingConfiguration(AWSObject):
    """
    `MatchmakingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-matchmakingconfiguration.html>`__
    """

    resource_type = "AWS::GameLift::MatchmakingConfiguration"

    props: PropsDictType = {
        "AcceptanceRequired": (boolean, True),
        "AcceptanceTimeoutSeconds": (integer, False),
        "AdditionalPlayerCount": (integer, False),
        "BackfillMode": (str, False),
        "CreationTime": (str, False),
        "CustomEventData": (str, False),
        "Description": (str, False),
        "FlexMatchMode": (str, False),
        "GameProperties": ([GameProperty], False),
        "GameSessionData": (str, False),
        "GameSessionQueueArns": ([str], False),
        "Name": (str, True),
        "NotificationTarget": (str, False),
        "RequestTimeoutSeconds": (integer, True),
        "RuleSetArn": (str, False),
        "RuleSetName": (str, True),
        "Tags": (Tags, False),
    }


class MatchmakingRuleSet(AWSObject):
    """
    `MatchmakingRuleSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-matchmakingruleset.html>`__
    """

    resource_type = "AWS::GameLift::MatchmakingRuleSet"

    props: PropsDictType = {
        "Name": (str, True),
        "RuleSetBody": (str, True),
        "Tags": (Tags, False),
    }


class S3Location(AWSProperty):
    """
    `S3Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-script-s3location.html>`__
    """

    props: PropsDictType = {
        "Bucket": (str, True),
        "Key": (str, True),
        "ObjectVersion": (str, False),
        "RoleArn": (str, True),
    }


class Script(AWSObject):
    """
    `Script <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-script.html>`__
    """

    resource_type = "AWS::GameLift::Script"

    props: PropsDictType = {
        "Name": (str, False),
        "StorageLocation": (S3Location, True),
        "Tags": (Tags, False),
        "Version": (str, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# Copyright (c) 2014, Andy Botting <andy.botting@theguardian.com>
# All rights reserved.
#
# See LICENSE file for full license.


from troposphere import AWSObject, AWSProperty
from troposphere.validators import (
    boolean,
    integer,
    integer_range,
    network_port,
    positive_integer,
)


class Firewall(AWSObject):
    resource_type = "OS::Neutron::Firewall"

    props = {
        "admin_state_up": (boolean, False),
        "description": (str, False),
        "firewall_policy_id": (str, True),
        "name": (str, False),
    }


class FirewallPolicy(AWSObject):
    resource_type = "OS::Neutron::FirewallPolicy"

    props = {
        "audited": (boolean, False),
        "description": (str, False),
        "firewall_rules": (list, True),
        "name": (str, False),
        "shared": (boolean, False),
    }


class FirewallRule(AWSObject):
    resource_type = "OS::Neutron::FirewallRule"

    props = {
        "action": (str, False),
        "description": (str, False),
        "destination_ip_address": (str, False),
        "destination_port": (network_port, False),
        "enabled": (boolean, False),
        "ip_version": (str, False),
        "name": (str, False),
        "protocol": (str, False),
        "shared": (boolean, False),
        "source_ip_address": (str, False),
        "source_port": (network_port, False),
    }

    def validate(self):
        if "action" in self.resource:
            action = self.resource["action"]
            if action not in ["allow", "deny"]:
                raise ValueError("The action attribute must be " "either allow or deny")

        if "ip_version" in self.resource:
            ip_version = self.resource["ip_version"]
            if ip_version not in ["4", "6"]:
                raise ValueError("The ip_version attribute must be " "either 4 or 6")

        if "protocol" in self.resource:
            protocol = self.resource["protocol"]
            if protocol not in ["tcp", "udp", "icmp", None]:
                raise ValueError(
                    "The protocol attribute must be " "either tcp, udp, icmp or None"
                )


class FloatingIP(AWSObject):
    resource_type = "OS::Neutron::FloatingIP"

    props = {
        "fixed_ip_address": (str, False),
        "floating_network_id": (str, True),
        "port_id": (str, False),
        "value_specs": (dict, False),
    }


class FloatingIPAssociation(AWSObject):
    resource_type = "OS::Neutron::FloatingIPAssociation"

    props = {
        "fixed_ip_address": (str, False),
        "floatingip_id": (str, True),
        "port_id": (str, False),
    }


class HealthMonitor(AWSObject):
    resource_type = "OS::Neutron::HealthMonitor"

    props = {
        "admin_state_up": (boolean, False),
        "delay": (positive_integer, True),
        "expected_codes": (str, False),
        "http_method": (str, False),
        "max_retries": (integer, True),
        "timeout": (integer, True),
        "type": (str, True),
        "url_path": (str, False),
    }

    def validate(self):
        if "type" in self.resource:
            mon_type = self.resource["type"]
            if mon_type not in ["PING", "TCP", "HTTP", "HTTPS"]:
                raise ValueError(
                    "The type attribute must be " "either PING, TCP, HTTP or HTTPS"
                )


class SessionPersistence(AWSProperty):
    props = {
        "cookie_name": (str, False),
        "type": (str, False),
    }

    def validate(self):
        if "type" in self.resource:
            if "cookie_name" not in self.resource:
                raise ValueError(
                    "The cookie_name attribute must be "
                    "given if session type is APP_COOKIE"
                )

            session_type = self.resource["type"]
            if session_type not in ["SOURCE_IP", "HTTP_COOKIE", "APP_COOKIE"]:
                raise ValueError(
                    "The type attribute must be "
                    "either SOURCE_IP, HTTP_COOKIE or APP_COOKIE"
                )


class VIP(AWSProperty):
    props = {
        "address": (str, False),
        "admin_state_up": (boolean, False),
        "connection_limit": (integer, True),
        "description": (str, False),
        "name": (str, False),
        "protocol_port": (network_port, True),
        "session_persistence": (SessionPersistence, False),
    }


class Pool(AWSObject):
    resource_type = "OS::Neutron::Pool"

    props = {
        "admin_state_up": (boolean, False),
        "description": (str, False),
        "lb_method": (str, True),
        "monitors": (list, False),
        "name": (str, False),
        "protocol": (str, True),
        "subnet_id": (str, True),
        "vip": (VIP, False),
    }

    def validate(self):
        if "lb_method" in self.resource:
            lb_method = self.resource["lb_method"]
            if lb_method not in ["ROUND_ROBIN", "LEAST_CONNECTIONS", "SOURCE_IP"]:
                raise ValueError(
                    "The lb_method attribute must be "
                    "either ROUND_ROBIN, LEAST_CONNECTIONS "
                    "or SOURCE_IP"
                )

        if "protocol" in self.resource:
            protocol = self.resource["protocol"]
            if protocol not in ["TCP", "HTTP", "HTTPS"]:
                raise ValueError(
                    "The type attribute must be " "either TCP, HTTP or HTTPS"
                )


class LoadBalancer(AWSObject):
    resource_type = "OS::Neutron::LoadBalancer"

    props = {
        "members": (list, False),
        "pool_id": (Pool, True),
        "protocol_port": (network_port, True),
    }


class Net(AWSObject):
    resource_type = "OS::Neutron::Net"

    props = {
        "admin_state_up": (boolean, False),
        "name": (str, False),
        "shared": (boolean, False),
        "tenant_id": (str, False),
        "value_specs": (dict, False),
    }


class PoolMember(AWSObject):
    resource_type = "OS::Neutron::PoolMember"

    props = {
        "address": (str, True),
        "admin_state_up": (boolean, False),
        "pool_id": (Pool, True),
        "protocol_port": (network_port, True),
        "weight": (integer_range(0, 256), False),
    }


class AddressPair(AWSProperty):
    props = {
        "ip_address": (str, True),
        "mac_address": (str, False),
    }


class FixedIP(AWSProperty):
    props = {
        "ip_address": (str, False),
        "subnet_id": (str, False),
    }


class Port(AWSObject):
    resource_type = "OS::Neutron::Port"

    props = {
        "admin_state_up": (boolean, False),
        "allowed_address_pairs": (list, False),
        "device_id": (str, False),
        "fixed_ips": (list, False),
        "mac_address": (str, False),
        "name": (str, False),
        "network_id": (str, True),
        "security_groups": (list, False),
        "value_specs": (dict, False),
    }


class SecurityGroup(AWSObject):
    resource_type = "OS::Neutron::SecurityGroup"

    props = {
        "description": (str, True),
        "name": (str, False),
        "rules": (list, False),
    }


class SecurityGroupRule(AWSProperty):
    props = {
        "direction": (str, False),
        "ethertype": (str, False),
        "port_range_max": (network_port, False),
        "port_range_min": (network_port, False),
        "protocol": (str, False),
        "remote_group_id": (str, False),
        "remote_ip_prefix": (str, False),
        "remote_mode": (str, False),
    }

    def validate(self):
        if "direction" in self.resource:
            direction = self.resource["direction"]
            if direction not in ["ingress", "egress"]:
                raise ValueError(
                    "The direction attribute must be " "either ingress or egress"
                )

        if "ethertype" in self.resource:
            ethertype = self.resource["ethertype"]
            if ethertype not in ["IPv4", "IPv6"]:
                raise ValueError(
                    "The ethertype attribute must be " "either IPv4 or IPv6"
                )

        if "protocol" in self.resource:
            protocol = self.resource["protocol"]
            if protocol not in ["tcp", "udp", "icmp"]:
                raise ValueError(
                    "The protocol attribute must be " "either tcp, udp or icmp"
                )

        if "remote_mode" in self.resource:
            remote_mode = self.resource["remote_mode"]
            if remote_mode not in ["remote_ip_prefix", "remote_group_id"]:
                raise ValueError(
                    "The remote_mode attribute must be "
                    "either remote_ip_prefix or remote_group_id"
                )



import collections
import importlib
import pkgutil
import unittest

from troposphere import AWSObject


class TestIntTypeShouldNotBeUsed(unittest.TestCase):
    def test_there_should_not_be_any_awsobject_with_int_in_props(self):
        """
        Anything that requires an integer should use `validators.integer`
        rather than `int`, because `int` does not work with types
        that behave like an integer (i.e. `long` in Python 2).
        """
        aws_objects = self.get_aws_objects()
        for obj in aws_objects:
            self.assertNoIntType(obj)

    def get_aws_objects(self):
        result = []
        modules = self._import_all_modules()
        for module in modules:
            for name in dir(module):
                obj = getattr(module, name)
                if (
                    isinstance(obj, type)
                    and obj != AWSObject
                    and issubclass(obj, AWSObject)
                ):
                    result.append(obj)
        return result

    def assertNoIntType(self, obj):
        for prop_name, (types, required) in obj.props.items():
            error_msg = (
                "{}.props['{}'] should have `validators.integer` "
                "rather than `int`".format(obj.__name__, prop_name)
            )
            if isinstance(types, collections.abc.Iterable):
                assert int not in types, error_msg
            else:
                assert types != int, error_msg

    def _get_all_troposphere_modules(self, dirname):
        return [
            pkg_name
            for importer, pkg_name, is_pkg in pkgutil.walk_packages([dirname])
            if not is_pkg and pkg_name.startswith("troposphere")
        ]

    def _import_all_modules(self):
        module_names = self._get_all_troposphere_modules(".")
        return [importlib.import_module(m) for m in module_names]



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_validity_type(validity_type):
    """
    Certificate Validity Type validation rule.
    Property: Validity.Type
    """
    VALID_VALIDITY_TYPE = ("ABSOLUTE", "DAYS", "END_DATE", "MONTHS", "YEARS")

    if validity_type not in VALID_VALIDITY_TYPE:
        raise ValueError(
            "Certificate Validity Type must be one of: %s"
            % ", ".join(VALID_VALIDITY_TYPE)
        )
    return validity_type


def validate_signing_algorithm(signing_algorithm):
    """
    Certificate SigningAlgorithm validation rule.
    Property: Certificate.SigningAlgorithm
    Property: CertificateAuthority.SigningAlgorithm
    """

    VALID_SIGNIN_ALGORITHM = [
        "SHA256WITHECDSA",
        "SHA256WITHRSA",
        "SHA384WITHECDSA",
        "SHA384WITHRSA",
        "SHA512WITHECDSA",
        "SHA512WITHRSA",
    ]

    if signing_algorithm not in VALID_SIGNIN_ALGORITHM:
        raise ValueError(
            "Certificate SigningAlgorithm must be one of: %s"
            % ", ".join(VALID_SIGNIN_ALGORITHM)
        )
    return signing_algorithm


def validate_key_algorithm(key_algorithm):
    """
    CertificateAuthority KeyAlgorithm validation rule.
    Property: CertificateAuthority.KeyAlgorithm
    """

    VALID_KEY_ALGORITHM = ("EC_prime256v1", "EC_secp384r1", "RSA_2048", "RSA_4096")

    if key_algorithm not in VALID_KEY_ALGORITHM:
        raise ValueError(
            "CertificateAuthority KeyAlgorithm must be one of: %s"
            % ", ".join(VALID_KEY_ALGORITHM)  # NOQA
        )
    return key_algorithm


def validate_certificateauthority_type(certificateauthority_type):
    """
    CertificateAuthority Type validation rule.
    Property: CertificateAuthority.Type
    """

    VALID_CERTIFICATEAUTHORITY_TYPE = ("ROOT", "SUBORDINATE")

    if certificateauthority_type not in VALID_CERTIFICATEAUTHORITY_TYPE:
        raise ValueError(
            "CertificateAuthority Type must be one of: %s"
            % ", ".join(VALID_CERTIFICATEAUTHORITY_TYPE)
        )
    return certificateauthority_type



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, integer
from .validators.wafregional import validate_waf_action_type


class FieldToMatch(AWSProperty):
    """
    `FieldToMatch <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-xssmatchset-fieldtomatch.html>`__
    """

    props: PropsDictType = {
        "Data": (str, False),
        "Type": (str, True),
    }


class ByteMatchTuples(AWSProperty):
    """
    `ByteMatchTuples <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-bytematchtuple.html>`__
    """

    props: PropsDictType = {
        "FieldToMatch": (FieldToMatch, True),
        "PositionalConstraint": (str, True),
        "TargetString": (str, False),
        "TargetStringBase64": (str, False),
        "TextTransformation": (str, True),
    }


class ByteMatchSet(AWSObject):
    """
    `ByteMatchSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-bytematchset.html>`__
    """

    resource_type = "AWS::WAFRegional::ByteMatchSet"

    props: PropsDictType = {
        "ByteMatchTuples": ([ByteMatchTuples], False),
        "Name": (str, True),
    }


class GeoMatchConstraints(AWSProperty):
    """
    `GeoMatchConstraints <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-geomatchset-geomatchconstraint.html>`__
    """

    props: PropsDictType = {
        "Type": (str, True),
        "Value": (str, True),
    }


class GeoMatchSet(AWSObject):
    """
    `GeoMatchSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-geomatchset.html>`__
    """

    resource_type = "AWS::WAFRegional::GeoMatchSet"

    props: PropsDictType = {
        "GeoMatchConstraints": ([GeoMatchConstraints], False),
        "Name": (str, True),
    }


class IPSetDescriptors(AWSProperty):
    """
    `IPSetDescriptors <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-ipset-ipsetdescriptor.html>`__
    """

    props: PropsDictType = {
        "Type": (str, True),
        "Value": (str, True),
    }


class IPSet(AWSObject):
    """
    `IPSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ipset.html>`__
    """

    resource_type = "AWS::WAFRegional::IPSet"

    props: PropsDictType = {
        "IPSetDescriptors": ([IPSetDescriptors], False),
        "Name": (str, True),
    }


class Predicates(AWSProperty):
    """
    `Predicates <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-ratebasedrule-predicate.html>`__
    """

    props: PropsDictType = {
        "DataId": (str, True),
        "Negated": (boolean, True),
        "Type": (str, True),
    }


class RateBasedRule(AWSObject):
    """
    `RateBasedRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ratebasedrule.html>`__
    """

    resource_type = "AWS::WAFRegional::RateBasedRule"

    props: PropsDictType = {
        "MatchPredicates": ([Predicates], False),
        "MetricName": (str, True),
        "Name": (str, True),
        "RateKey": (str, True),
        "RateLimit": (integer, True),
    }


class RegexPatternSet(AWSObject):
    """
    `RegexPatternSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-regexpatternset.html>`__
    """

    resource_type = "AWS::WAFRegional::RegexPatternSet"

    props: PropsDictType = {
        "Name": (str, True),
        "RegexPatternStrings": ([str], True),
    }


class Rule(AWSObject):
    """
    `Rule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html>`__
    """

    resource_type = "AWS::WAFRegional::Rule"

    props: PropsDictType = {
        "MetricName": (str, True),
        "Name": (str, True),
        "Predicates": ([Predicates], False),
    }


class SizeConstraint(AWSProperty):
    """
    `SizeConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-sizeconstraint.html>`__
    """

    props: PropsDictType = {
        "ComparisonOperator": (str, True),
        "FieldToMatch": (FieldToMatch, True),
        "Size": (integer, True),
        "TextTransformation": (str, True),
    }


class SizeConstraintSet(AWSObject):
    """
    `SizeConstraintSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sizeconstraintset.html>`__
    """

    resource_type = "AWS::WAFRegional::SizeConstraintSet"

    props: PropsDictType = {
        "Name": (str, True),
        "SizeConstraints": ([SizeConstraint], False),
    }


class SqlInjectionMatchTuples(AWSProperty):
    """
    `SqlInjectionMatchTuples <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sqlinjectionmatchset-sqlinjectionmatchtuple.html>`__
    """

    props: PropsDictType = {
        "FieldToMatch": (FieldToMatch, True),
        "TextTransformation": (str, True),
    }


class SqlInjectionMatchSet(AWSObject):
    """
    `SqlInjectionMatchSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sqlinjectionmatchset.html>`__
    """

    resource_type = "AWS::WAFRegional::SqlInjectionMatchSet"

    props: PropsDictType = {
        "Name": (str, True),
        "SqlInjectionMatchTuples": ([SqlInjectionMatchTuples], False),
    }


class Action(AWSProperty):
    """
    `Action <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-webacl-action.html>`__
    """

    props: PropsDictType = {
        "Type": (validate_waf_action_type, True),
    }


class Rules(AWSProperty):
    """
    `Rules <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-webacl-rule.html>`__
    """

    props: PropsDictType = {
        "Action": (Action, True),
        "Priority": (integer, True),
        "RuleId": (str, True),
    }


class WebACL(AWSObject):
    """
    `WebACL <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html>`__
    """

    resource_type = "AWS::WAFRegional::WebACL"

    props: PropsDictType = {
        "DefaultAction": (Action, True),
        "MetricName": (str, True),
        "Name": (str, True),
        "Rules": ([Rules], False),
    }


class WebACLAssociation(AWSObject):
    """
    `WebACLAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webaclassociation.html>`__
    """

    resource_type = "AWS::WAFRegional::WebACLAssociation"

    props: PropsDictType = {
        "ResourceArn": (str, True),
        "WebACLId": (str, True),
    }


class XssMatchTuple(AWSProperty):
    """
    `XssMatchTuple <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-xssmatchset-xssmatchtuple.html>`__
    """

    props: PropsDictType = {
        "FieldToMatch": (FieldToMatch, True),
        "TextTransformation": (str, True),
    }


class XssMatchSet(AWSObject):
    """
    `XssMatchSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-xssmatchset.html>`__
    """

    resource_type = "AWS::WAFRegional::XssMatchSet"

    props: PropsDictType = {
        "Name": (str, True),
        "XssMatchTuples": ([XssMatchTuple], False),
    }
